"""Example 23 modified for minimization."""

from Definitions import *

# . Header.
logFile.Header ( )

# . Define some parameters.
_DIncrement    =   1.0
_DMinimum      =   1.5
_DName         = "dOH"
_ForceConstant =  20.0
_Windows       =     5

# . Define the atom indices.
_Oxygen   =  5
_Hydrogen = 17

# . Define the MM and NB models.
mmModel = MMModelOPLS.WithParameterSet ( "bookSmallExamples" )
nbModel = NBModelFull.WithDefaults ( )

# . Generate the molecule.
molecule = ImportSystem ( os.path.join ( molPath, "bala_c7eq.mol" ) )
molecule.DefineMMModel ( mmModel )
molecule.DefineNBModel ( nbModel )
molecule.Summary ( )

# . Read in the starting coordinates.
molecule.coordinates3 = ImportCoordinates3 ( os.path.join ( xyzPath, "bala_1pt5.xyz" ) )

# . Define an empty restraint model and assign it to the system.
restraints = RestraintModel ( )
molecule.DefineRestraintModel ( restraints )

results = []

# . Loop over the values of the distance.
for i in range ( _Windows ):

    # . Calculate the new restraint distance.
    distance = _DIncrement * float ( i ) + _DMinimum

    # . Define a new restraint.
    rModel    = RestraintEnergyModel.Harmonic ( distance, _ForceConstant )
    restraint = RestraintDistance.WithOptions ( energyModel = rModel, point1 = _Oxygen, point2 = _Hydrogen )
    restraints[_DName] = restraint

    # . Optimization.
    ConjugateGradientMinimize_SystemGeometry ( molecule                    ,
                                               logFrequency         =  100 ,
                                               maximumIterations    = 2000 ,
                                               rmsGradientTolerance =  0.1 )

    # . Determine the final energy.
    eStop = molecule.Energy ( )
    dStop = molecule.coordinates3.Distance ( _Oxygen , _Hydrogen )
    results.append ( (distance, dStop, eStop) )

# . Print the results.
table = logFile.GetTable ( columns = [ 30, 30, 30 ] )
table.Start ( )
table.Title ( "Minimization Results" )
table.Heading ( "Restrain (OH)" )
table.Heading ( "Distance (OH)" )
table.Heading ( "Energy " )
for result in results:
   table.Entry ( "{:20.4f}".format ( result[0] ) )
   table.Entry ( "{:20.4f}".format ( result[1] ) )
   table.Entry ( "{:20.4f}".format ( result[2] ) )
table.Stop ( )

# . Footer.
logFile.Footer ( )
